# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Example resource."""

from invenio_records_resources.resources import \
    FileResourceConfig as FileResourceConfigBase
from marshmallow import Schema

from invenio_drafts_resources.resources import RecordResourceConfig


class RecordResourceConfig(RecordResourceConfig):
    """Mock record resource configuration."""

    blueprint_name = "mocks"
    url_prefix = "/mocks"


class FileResourceConfig(FileResourceConfigBase):
    """Mock record file resource."""

    blueprint_name = "mocks_files"
    url_prefix = "/mocks/<pid_value>"


class DraftFileResourceConfig(FileResourceConfigBase):
    """Mock record file resource."""

    blueprint_name = "mocks_draft_files"
    url_prefix = "/mocks/<pid_value>/draft"



"""Example of a record draft API."""

from invenio_records.systemfields import ConstantField, ModelField
from invenio_records_resources.records import FileRecord as FileRecordBase
from invenio_records_resources.records.systemfields import FilesField, \
    IndexField

from invenio_drafts_resources.records import Draft as DraftBase
from invenio_drafts_resources.records import ParentRecord as ParentRecordBase
from invenio_drafts_resources.records import Record as RecordBase

from .models import DraftMetadata, FileDraftMetadata, FileRecordMetadata, \
    ParentRecordMetadata, ParentState, RecordMetadata


class ParentRecord(ParentRecordBase):
    """Example parent record."""

    # Configuration
    model_cls = ParentRecordMetadata

    # System fields
    schema = ConstantField(
        '$schema', 'http://localhost/schemas/records/parent-v1.0.0.json')


class FileRecord(FileRecordBase):
    """Example record file API."""

    model_cls = FileRecordMetadata
    record_cls = None  # defined below


class Record(RecordBase):
    """Example record API."""

    # Configuration
    model_cls = RecordMetadata
    versions_model_cls = ParentState
    parent_record_cls = ParentRecord

    # System fields
    schema = ConstantField(
        '$schema', 'http://localhost/schemas/records/record-v1.0.0.json')

    index = IndexField(
        'draftsresources-records-record-v1.0.0',
        search_alias='draftsresources-records'
    )

    files = FilesField(
        store=False,
        file_cls=FileRecord,
        # Don't create
        create=False,
        # Don't delete, we'll manage in the service
        delete=False,
    )

    bucket_id = ModelField(dump=False)

    bucket = ModelField(dump=False)


class FileDraft(FileRecordBase):
    """Example record file API."""

    model_cls = FileDraftMetadata
    record_cls = None  # defined below


class Draft(DraftBase):
    """Example record API."""

    # Configuration
    model_cls = DraftMetadata
    versions_model_cls = ParentState
    parent_record_cls = ParentRecord

    # System fields
    schema = ConstantField(
        '$schema', 'http://localhost/schemas/records/record-v1.0.0.json')

    index = IndexField(
        'draftsresources-drafts-draft-v1.0.0',
        search_alias='draftsresources-drafts'
    )

    files = FilesField(
        store=False,
        file_cls=FileDraft,
        # Don't delete, we'll manage in the service
        delete=False,
    )

    bucket_id = ModelField(dump=False)

    bucket = ModelField(dump=False)


FileRecord.record_cls = Record
FileDraft.record_cls = Draft



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""System fields."""

from .parent import ParentField
from .versions import VersionsField

__all__ = (
    "ParentField",
    "VersionsField",
)



"""Example of a record model."""

from invenio_db import db
from invenio_records.models import RecordMetadataBase
from invenio_records_resources.records import FileRecordModelMixin

from invenio_drafts_resources.records import DraftMetadataBase, \
    ParentRecordMixin, ParentRecordStateMixin


class ParentRecordMetadata(db.Model, RecordMetadataBase):
    """Model for mock module metadata."""

    __tablename__ = 'parent_mock_metadata'


class DraftMetadata(db.Model, DraftMetadataBase, ParentRecordMixin):
    """Model for mock module metadata."""

    __tablename__ = 'draft_mock_metadata'
    __parent_record_model__ = ParentRecordMetadata


class RecordMetadata(db.Model, RecordMetadataBase, ParentRecordMixin):
    """Model for mock module metadata."""

    __tablename__ = 'record_mock_metadata'
    __parent_record_model__ = ParentRecordMetadata


class ParentState(db.Model, ParentRecordStateMixin):
    """Model for mock module for parent state."""

    __parent_record_model__ = ParentRecordMetadata
    __record_model__ = RecordMetadata
    __draft_model__ = DraftMetadata


class FileRecordMetadata(db.Model, RecordMetadataBase, FileRecordModelMixin):
    """Model for mock module record files."""

    __record_model_cls__ = RecordMetadata

    __tablename__ = 'mock_record_files'


class FileDraftMetadata(db.Model, RecordMetadataBase, FileRecordModelMixin):
    """Model for mock module draft files."""

    __record_model_cls__ = DraftMetadata

    __tablename__ = 'mock_draft_files'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft aware Record Resource Config override."""

from invenio_records_resources.resources import \
    RecordResourceConfig as RecordResourceConfigBase

from .args import SearchRequestArgsSchema


class RecordResourceConfig(RecordResourceConfigBase):
    """Draft aware Record resource config."""

    url_prefix = ""

    routes = {
        "user-prefix": "/user",
        "list": "",
        "item": "/<pid_value>",
        "item-versions": "/<pid_value>/versions",
        "item-latest": "/<pid_value>/versions/latest",
        "item-draft": "/<pid_value>/draft",
        "item-publish": "/<pid_value>/draft/actions/publish",
        "item-files-import": "/<pid_value>/draft/actions/files-import",
    }

    request_search_args = SearchRequestArgsSchema



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Data layer API.

The 'records' folder name stands for the generic records-resource's data layer.
It applies to both api.py's Draft and Record.
"""

from .api import Draft, ParentRecord, Record
from .models import DraftMetadataBase, ParentRecordMixin, \
    ParentRecordStateMixin

__all__ = (
    "Draft",
    "DraftMetadataBase",
    "ParentRecord",
    "ParentRecordMixin",
    "ParentRecordStateMixin",
    "Record",
)
